//=========================================================================
//
//	FILE:				CP_ErrorManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle error reporting in the CPLAT framework.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 199 $
//						Latest Checkout:  $Date: 2005-06-02 14:27:57 -0700 (Thu, 02 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_ErrorManager
#define _H_CP_ErrorManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_NoCopy.h"
#include "CP_String.h"

	// Standard Library
#include <exception>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_ErrorManager
*	@brief		A class to handle reporting of all error conditions in a CPLAT application. 
*	
*				This is the single class that is responsible for all error
*				reporting in CPLAT. You can set the actual routine that gets called
*				when an error is reported to your own, or use the default error reporting
*				routines which display an alert to the user.
*/
class CP_EXPORT CP_ErrorManager : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ErrorManager();
	//! Destructor.
	virtual								~CP_ErrorManager();
	//@}

	//! @name Instance
	//@{
	static	CP_ErrorManager				*Instance();
	//@}
	
	//! @name Exceptions
	//@{
			void						ReportException(
												const CP_String& inMessage,
												const std::exception& inException,
												const char *inFunction,
												int inLine,
												const char *inFile );
	//@}

	//! @name Errors
	//@{
			void						ReportError(
												const CP_String& inMessage,
												const char *inFunction,
												int inLine,
												const char *inFile );
			void						ReportError(
												const CP_String& inMessage,
												const CP_String& inDescription,
												const char *inFunction,
												int inLine,
												const char *inFile );
	//@}

	//! @name Warnings
	//@{
			void						ReportWarning(
												const CP_String& inMessage,
												const char *inFunction,
												int inLine,
												const char *inFile );
			void						ReportWarning(
												const CP_String& inMessage,
												const CP_String& inDescription,
												const char *inFunction,
												int inLine,
												const char *inFile );
	//@}

	//! @name Notifications
	//@{
			void						SurpressAlertNotifications( CP_Debugging::CP_E_DebugNotification inNotifiction = CP_Debugging::CP_E_DebugNotification_Nothing );
			void						RestoreAlertNotifications();
	//@}
			
protected:

	// class data
private:
	static CP_ErrorManager					*fsInstance;

	CP_Debugging::CP_E_DebugNotification	fSavedExceptionNotification;
	CP_Debugging::CP_E_DebugNotification	fSavedErrorNotification;
	CP_Debugging::CP_E_DebugNotification	fSavedWarningNotification;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Reporting
	//@{
			void						ReportError(
												const CP_String& inMessage,
												const CP_String& inDescription );
			void						ReportException_OS(
												const CP_String& inMessage,
												const std::exception& inException,
												const char *inFunction,
												int inLine,
												const char *inFile );
			void						ReportWarning(
												const CP_String& inMessage,
												const CP_String& inDescription );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Reporting
	//@{
			void						ReportError(
												const CP_String& inMessage,
												const CP_String& inDescription );
			void						ReportException_OS(
												const CP_String& inMessage,
												const std::exception& inException,
												const char *inFunction,
												int inLine,
												const char *inFile );
			void						ReportWarning(
												const CP_String& inMessage,
												const CP_String& inDescription );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Reporting
	//@{
			void						ReportError(
												const CP_String& inMessage,
												const CP_String& inDescription );
			void						ReportException_OS(
												const CP_String& inMessage,
												const std::exception& inException,
												const char *inFunction,
												int inLine,
												const char *inFile );
			void						ReportWarning(
												const CP_String& inMessage,
												const CP_String& inDescription );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ErrorManager
