//=========================================================================
//
//	FILE:				CP_PtrBlock.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Allocates blocks of memory using pointers.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_PtrBlock
#define _H_CP_PtrBlock
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_PtrBlock
*	@brief		Memory allocation class based on pointers.
*	
*				A reference counted object to manage blocks of memory using pointers.
*/
class CP_EXPORT CP_PtrBlock
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_PtrBlock();
										explicit CP_PtrBlock( UInt32 inSize );
										explicit CP_PtrBlock( CP_PtrBlock& inPtrBlock );
	//! Destructor.
										~CP_PtrBlock();
	//@}

	//! @name Copying
	//@{
				void					CopyTo(
												void *ioBuffer,
												UInt32 inNumBytes ) const;
				void					CopyFrom(
												const void *inBuffer,
												UInt32 inNumBytes );
	//@}

	//! @name Getters
	//@{
										/*! Geturn the memory block.*/
				OSPtr					GetOSPtr()	const								{ return( fBlock ); }
				char					*GetPtr() const;
	//@}

	//! @name Length
	//@{
				UInt32					Length() const;
				void					SetLength( UInt32 inLength );
	//@}

	//! @name Locking
	//@{
				void					Lock() const;
				void					Unlock() const;

										/*!
										*	Returns true if the memory block is locked.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is locked.
										*/
				bool					IsLocked()	const								{ return( fLocked ); }
	//@}

	//! @name Operators
	//@{
			CP_PtrBlock&				operator = (const CP_PtrBlock& inPtrBlock );
	//@}
				
	//! @name Testers
	//@{
										/*!
										*	Returns true if the memory block is not NULL.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is not NULL.
										*/
				bool					IsValid() const								{ return( fBlock != NULL ); }

										/*!
										*	Returns true if this object owns the block of memory, and is therefore
										*	responsible for its deletion. Otherwise the memory block held by this object
										*	will not be deleted by this class.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if this object owns the memory handle.
										*/
				bool					OwnsPtr() const								{ return( fOwnsPtr ); }
	//@}

	//! @name Wrapping
	//@{
				void					Wrap(
											OSPtr inExistingPtr,
											bool inOwnsPtr = true );

										/*!
										*	Notifies this class that it no longer owns the memory, and will not
										*	delete it. You are now responsible for deleting the memory to avoid a
										*	memory leak.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
				void					Detach()									{ fOwnsPtr = false; }
	//@}

private:
	//! @name Allocation
	//@{
				void					Allocate( UInt32 inSize );
				void					Deallocate();
	//@}

	// class data
protected:
	OSPtr								fBlock;					/*! Platforms memory object.*/
	bool								fOwnsPtr;				/*! Flag to indicate who is responsible for deleting the platforms memory.*/
	mutable bool						fLocked;				/*! Flag to indicate if the block is locked.*/
								
	UInt32								fActualBlockLength;		/*! Length of the memory block.*/

#if TARGET_OS_WIN32
	mutable char						*fBlockPtr;				/*! Pointer to the memory when the block is locked.*/
#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
							  explicit CP_PtrBlock(
												Ptr inMacPtr, 
												bool inOwnsPtr = true );
	//@}

	//! @name operators
	//@{
										/*!
										*	Returns the MacOS Handle.
										*	
										*	@exception	none
										*	
										*	@return		OSHandle
										*/
										operator Ptr() const								{ return( fBlock ); }
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_PtrBlock



