//=========================================================================
//
//	FILE:				CP_TimedTask.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to call a user slot at specified time intervals.
//						The time intervals are expressed in milliseconds.
//
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 160 $
//						Latest Checkout:  $Date: 2005-04-27 21:21:26 -0700 (Wed, 27 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_TimedTask
#define _H_CP_TimedTask
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Object.h"
#include "CP_SignalSlotHolder.h"
#if TARGET_OS_WIN32
#include "CP_Event.h"
#include "CP_Window.h"
#include "CP_Mutex.h"
#endif

CPLAT_Begin_Namespace_CPLAT

#define kTimerDuration_MilliSec				1
#define kTimerDuration_Sec					1000

/*!
*	@class		CP_TimedTask
*	@brief		A class to call a user slot at specified time intervals.
*	
*				This classs is used to call a user defined slot at a specified
*				time interval, measured in milliseconds.
*
*				Example usage:
*
*	@code
*				CPLAT::CP_TimedTask				*timer = CP_new CPLAT::CP_TimedTask( kEventDurationSecond * 5, dataValue );
*	
*				timer->FindSignal( kCP_Signal_TimedTask )->connect( CP_SigSlot::slot(this, &Timer_Application::TimerTask) );
*	
*				timer->Start();
*	@endcode
*
*				In this above example, the slot TimerTask will be called every 5 seconds. The dataValue passed to the CP_TimedTask
*				constructor will be passed to the timer slot you have defined. You can use this data
*				member anyway you want.
*
*/
class CP_EXPORT CP_TimedTask : public CP_SigSlot::CP_SlotObject, public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TimedTask(
												double inMilliSecFrequency,
												UInt32 inUserData = 0 );
	//! Destructor.
	virtual								~CP_TimedTask() throw();
	//@}
	
			void						Start();
			
			void						Stop();
					
	virtual	void						HandleTask();

										/*!
										*	Returns the flag that indicates if this timer is currently running.
										*	
										*	@exception	none
										*	
										*	@return		bool			True if the timer is currently running.
										*/
			bool						IsRunning()									{ return( fIsRunning ); }

protected:

	// class data
protected:
	bool								fIsRunning;				/* ! Flag to indicate if the timer is running. */
	UInt32 								fUserData;				/* ! User data passed to the timer slot. */
	
	CP_DataValueSignal					fTimerTaskSignal;		/* ! Signal called when the timer fires. */
	
	double								fMilliSecFrequency;		/* ! Frequency in milliseconds that the timer fires. */

	#if TARGET_OS_MAC
	# if CP_CORE_ONLY
		CFRunLoopRef					fRunLoop;				/* ! Carbon run loop we're installed in. */
		CFRunLoopTimerRef				fTimer;				/* ! Carbon run loop timer ref. */
	# else
		EventLoopTimerRef				fTimer;				/* ! Carbon event loop timer ref. */
	# endif
	#endif

	#if TARGET_OS_WIN32
	# if !CP_CORE_ONLY
		CP_Window					*fTimerWindow;		/* ! Window that will receive timer calls. Note: This window is never visible. */
	# else
		static std::map<UINT, CP_TimedTask *> fRegisteredTimers; // keep a map with registered timers to quick get to the CP_TimedTask object pointer
		static CP_Mutex fMapMutex;                               //<! to serialize the access to the registered timers map
		static void CALLBACK TimerProcedure( HWND hWnd, UINT nMsg, UINT_PTR idEvent, DWORD dwTime );
	# endif
		UINT							fWinTimer;				/* ! Windows timer id. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#if CP_CORE_ONLY
protected:
			static void _CFRunLoopTimerCallback(
								CFRunLoopTimerRef inTimer, 
								void* inInfo);
#else
protected:
			EventLoopTimerUPP					GetTimerUPP();
		
private:
	static pascal void					CP_TimedTask_timerCallback(
												EventLoopTimerRef inTimer,
												void *userData );
#endif
#endif

#if TARGET_OS_WIN32
private:
		#if !CP_CORE_ONLY
		void 							CreateDummyTimerWindow();
		#endif
		void							HandlerWMTimer( CPLAT::CP_Event_Base& inEvent );
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TimedTask

