//=========================================================================
//
//	FILE:				CP_UUID.h
//	
//						Created by Jens Miltner on 10.01.05.
//	
//	PURPOSE:			CP_PropertyList handling.
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 107 $
//						Latest Checkout:  $Date: 2005-03-21 15:11:37 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_UUID
#define _H_CP_UUID

// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

struct CP_UUIDBytes
{
	UInt8	bytes[16];
};

class CP_String;

class CP_UUID
{
public:
	static const CP_UUID	null; //!< the 'null' UUID consisting of only zero bytes
	
	//! @name Constructor/Destructor
	//! @{
	//! Constructor.
	CP_UUID();
	CP_UUID(const CP_UUID& inSrc);
	CP_UUID(const CP_UUIDBytes& inUUIDBytes);
	CP_UUID(const CP_String& inUUIDString);
	//! Destructor.
	~CP_UUID();
	//! @}
	
	/*!
	 *   @brief   Creates a unique ID.
	 *
	 *   Creates a new, unique UUID.
	 *
	 *   @exception CP_OSErrorException(coreFoundationUnknownErr) raised if CFUUID object can't be created (Mac).
	 *
	 *   @return     CP_UUID		The new, unique UUID.
	 */
	static CP_UUID Create();
	
	//! @name Getters
	//! @{
	/*!
	     @brief   Get the raw bytes forming the UUID.
	  
	     Copy the raw bytes that form the UUID. Useful to pass to functions
	  	 that expect the raw bytes or e.g. when storing UUIDs.
	  
	     @exception none
	  
	     @return     CP_UUIDBytes		The raw bytes forming this UUID.
	 */
	CP_UUIDBytes GetUUIDBytes() const;
	
	/*!
		 @brief   Get a string representation of the UUID.
	  
		 Constructs a string representation for the UUID in the standard format
	     (e.g. "A6B41150-63E8-11D9-99B7-000393D77230")
	  
		 @exception exceptions raised by CP_String
	  
		 @return     CP_String		The string representation of the UUID.
	 */
	CP_String GetString() const;
	//! @}
	
	//! @name comparison
	//! @{
	//! Check for equality
	bool operator == (const CP_UUID& inRhs) const;
	bool operator != (const CP_UUID& inRhs) const;
	//! Order relations (usefull when using UUIDs as key into a map, etc.)
	//! \note Order relations are consistent during runtime, but not necessarily between platforms.
	bool operator < (const CP_UUID& inRhs) const;
	bool operator <= (const CP_UUID& inRhs) const;
	bool operator > (const CP_UUID& inRhs) const;
	bool operator >= (const CP_UUID& inRhs) const;
	//! @}
	
	//! @name Assignment
	//! @{
	//! Assign a new value to this UUID.
	CP_UUID& operator = (const CP_UUID& inRhs);
	CP_UUID& operator = (const CP_UUIDBytes& inRhs);
	//! @}
	
protected:
	CP_UUIDBytes			fBytes; //!< the bytes that form the UUID

#if TARGET_OS_MAC
public:
	//! @name Constructor/Destructor
	//! @{
	//! Constructor.
	/*!
	 *   @brief   Construct a UUID object from a CoreFoundation UUID object reference.
	 *
	 *   Creates a CP_UUID object from a CoreFoundation UUID reference. 
	 *	 Retains the CF reference as needed.
	 *
	 *   @exception none
	 *
	 *   @param     inUUIDRef			A CoreFoundation UUID reference representing the UUID.
	 */
	CP_UUID(CFUUIDRef inUUIDRef);
	//! @}
	/*!
	 *   @brief   Get a CoreFoundation object for the UUID.
	 *
	 *   Returns a reference to a CoreFoundation UUID object for this ID. This object is owned by
	 *	 CP_UUID, so you need to retain it if you want to hold onto it.
	 *
	 *   @exception CP_OSErrorException(coreFoundationUnknownErr) raised if CoreFoundation object can't be allocated.
	 *
	 *   @return     inUUIDRef			A CoreFoundation UUID reference representing the UUID. 
	 */
	CFUUIDRef GetCFUUIDRef() const;
	
protected:
	/*!
	 *   @brief   Assert we have a CoreFoundation object for the UUID.
	 *
	 *   @exception CP_OSErrorException(coreFoundationUnknownErr) raised if CoreFoundation object can't be allocated.
	 */
	void AssertCFUUIDRef() const;
private:
	mutable CFUUIDRef		fUUIDRef; //!< the CoreFoundation object reference for this UUID
#endif
	
#if TARGET_OS_WIN32
	public:
		//! @name Constructor/Destructor
		//! @{
		//! Constructor.
		/*!
			*   @brief   Construct a UUID object from a Windows GUID object reference.
			*
			*   Creates a CP_UUID object from a Windows GUID reference. 
			*
			*   @exception none
			*
			*   @param     inSrc			A Windows GUID reference representing the UUID.
			*/
		CP_UUID(const GUID& inSrc);
		//! @}
		
		//! @name Getters
		//! @{
		/*!
				@brief   Get the GUID (Windows GUID) object
		  
				Get the information in a GUID (Windows GUID) form.
		  
				@exception none
		  
				@return     GUID: the Windows GUID
		*/
		GUID GetGUID() const;   
		//! @}
		
		//! @name Assignment
		//! @{
		//! @brief Assign a new value to this UUID from a Windows GUID
		CP_UUID& operator = (const GUID& rhs );
		//! @}

#endif
	
#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_UUID
