//=========================================================================
//
//	FILE:				CP_TableView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base view class for displaying data in a table format.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 204 $
//						Latest Checkout:  $Date: 2005-06-06 22:15:38 -0700 (Mon, 06 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_TableView
#define _H_CP_TableView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataSource.h"
#include "CP_TableGeometry_Base.h"
#include "CP_ScrollableView.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

enum
{
	kCP_TV_GeometryMono							= 0x0001,		// columns and rows the same
	kCP_TV_GeometryMulti						= 0x0002,		// columns and rows can be different, but same in each group
	kCP_TV_GeometryMixed						= 0x0004		// columns and rows can be different, including in each row
};

const SInt32 kCP_BadIndex = -1;

typedef std::vector<CP_Point>																CP_SelectedCells_List;
typedef std::vector<CP_Point>::iterator														CP_SelectedCells_Iterator;
typedef std::vector<CP_Point>::reverse_iterator												CP_SelectedCells_ReverseIterator;

class CP_EXPORT CP_TableView : public CP_ScrollableView
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TableView();
										CP_TableView(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												UInt32 inGeometry,
												bool inAllowMultiSelection = false );
	//@}
	//! Destructor.
	virtual								~CP_TableView() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Data Source
	//@{
			CP_DataSource				*GetDataSource()										{ return( fDataSource ); }
			void						SetDataSource( CP_DataSource *inDataSource );
	//@}

	//! @name Drawing
	//@{
	virtual	void						HiliteCell( const CP_Point& inCell );
	virtual	void						RefreshCell( const CP_Point& inCell );
	//@}

	//! @name Geometry
	//@{
			CP_TableGeometry_Base 		*GetTableGeometry()									{ return( fTableGeometry ); }
			void				 		SetTableGeometry( CP_TableGeometry_Base *inGeometry )	{ fTableGeometry = inGeometry; }
	//@}

	//! @name Selection
	//@{
			void						SelectCell( const CP_Point& inCell );
			void						UnselectCell( const CP_Point& inCell );
			
			void						SelectAllCells();
			void						UnselectAllCells();

			void						SelectRange(
												CP_Point& inStartCell,
												CP_Point& inEndCell,
												bool inSelect = true );

			bool						IsCellSelected( const CP_Point& inCell );
	virtual	SInt32						GetNumberSelections();
			SInt32						GetNumberSelectedCells();
			CP_Point					GetNthSelectedCell( SInt32 inIndex ) const;
			CP_SelectedCells_List		*GetSelectedCells()									{ return( fSelectedCells ); }
	//@}

	//! @name Sorting
	//@{
			void						SetAllowMultiSelection( bool inAllowMultiSelect )	{ fAllowMultiSelection = inAllowMultiSelect; }
	//@}

	//! @name Sorting
	//@{
			bool						GetSortUp()											{ return( fSortUp ); }
			void						SetSortUp( bool inSortUp )							{ fSortUp = inSortUp; }
	virtual	void						Sort();
	//@}

	//! @name Testing
	//@{
			bool						IsValidCell( const CP_Point& inCell );
	//@}

protected:
	//! @name Data Source
	//@{
	virtual	void						DataSourceChanged(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	//@}

	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	virtual	void						OnDrawCell(
												CP_Canvas& inCanvas,
												CP_Point& inCell,
												CP_Rect& inCellBounds );
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );

	virtual	void						HandleControlHitEvent( CP_Event_Base& inEvent );
			void						HandleRawKeyDownEvent( CPLAT::CP_Event_Base& inEvent );
	//@}

	//! @name Geometry
	//@{
	virtual	void						RowsChangedSlot(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	virtual	void						ColumnsChangedSlot(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	virtual	void						ColumnResizedSlot(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	virtual	void						RowHeightChangedSlot(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	virtual	void						ColumnWidthChangedSlot(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	virtual	void						ColumnSortSlot(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );

	virtual void						RecalcImageSize();
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	//! @name Selection
	//@{
			void						ExtendSelection(
												CP_Point& inAnchorCell,
												CP_Point& inHitCell );
	//@}

	// class data
protected:
	CP_TableGeometry_Base				*fTableGeometry;				/*! Contains information about each rows geometry in the table. */
	
	CP_DataSource						*fDataSource;					/*! Object used to supply data for the table. */

	CP_SelectedCells_List				*fSelectedCells;				/*! List of currently selected cells. */
	CP_Point							fAnchorCell;					/*! anchor cell for shift selections. */
	
	bool								fAllowMultiSelection;			/*! Flag to indicate if we allow multiple selections. */
	bool								fSortUp;						/*! Flag to indicate the sorting order. */
	
	CP_DataValueSignal					fClickedSignal;					/*! Signal sent when the user clicks on a cell. */
	CP_DataValueSignal					fDoubleClickedSignal;			/*! Signal sent when the user double clicks on a cell. */

	CP_DataValueSignal					fSelectionChangedSignal;		/*! Signal sent when the selection has changed. */

	CP_DataValueSignal					fContentHandlerDoneEditSignal;	/*! Signal sent when a content handler is done editing data. */

	CP_DataValueSlot					fColumnResizedSlot;				/*! Slot to handle something causing a column to be resized. */
	CP_DataValueSlot					fColumnSortSlot;				/*! Slot to handle something causing a column to sorted. */

		// Sorting	
	static SInt32						gSortCol;						/*! Holds the column being sorted. */
	static CP_TableView					*gSortTableView;				/*! Holds a pointer to ourselves. */
	static bool							gSortUp;						/*! Flag to indicate if we are sorting in ascending or descending order. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Events
	//@{
	virtual	ControlPartCode				HandleHitTest( const CP_Point& inMouseLocation );
	virtual	ControlPartCode				HandleTracking(
												const CP_Point& inMouseLocation,
												ControlActionUPP actionProc );

	virtual	ControlPartCode				HandleFocus( ControlFocusPart inFocusPart );
	virtual	void						HandleKeyDown(
												SInt16 keyCode,
												SInt16 charCode,
												SInt16 modifiers );
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TableView

