//=========================================================================
//
//	FILE:				CP_View_Imp_Base.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class for the actual implementation of the CP_View
//						object. There are implementations that can use a MacOS
//						Control, Windows child window, MacOS HIView.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 194 $
//						Latest Checkout:  $Date: 2005-05-25 22:20:49 -0700 (Wed, 25 May 2005) $
//
//=========================================================================
#ifndef _H_CP_View_Imp_Base
#define _H_CP_View_Imp_Base
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_DragDrop.h"
#include "CP_EventHandler.h"
#include "CP_M_CommandHandler.h"
#include "CP_Point.h"
#include "CP_Size.h"
#include "CP_Region.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_View;
class CP_View_Imp;

/*!
*	@class		CP_View_Mac_Imp
*	@brief		Base class for the actual implementation of the CP_View object.
*
*				There are implementations that can use a MacOS Control, Windows child
*				window, MacOS HIView.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_View_Imp_Base : public CP_M_CommandHandler
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_View_Imp_Base() throw();
	//@}
			
	//! @name Appearance
	//@{
	virtual	void						Show()												{}
	virtual	void						Hide()												{}

	virtual	void						Activate()											{}
	virtual	void						Deactivate()										{}

	virtual	void						Enable()											{}
	virtual	void						Disable()											{}
	//@}

	//! @name Coordinates
	//@{
	virtual	void						PortToLocal( CP_Point& inPortPoint );
			void						PortToLocal( CP_Rect& inPortRect );
			void						PortToGlobal( CP_Point& inPortPoint );
			void						PortToGlobal( CP_Rect& inPortRect );

			void						LocalToPort( CP_Point& inPortPoint );
			void						LocalToPort( CP_Rect& inPortRect );
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& /*inEventHandler*/ )	{}
	//@}

	//! @name Getters
	//@{
	virtual	CP_EventHandler&			GetEventHandler();

	virtual	CP_Rect						GetPortBounds();
			CP_Rect						GetLocalBounds();
	
			OSControl					GetOSControl() const;
			void						SetOSControl( OSControl inOSControl );
			CP_View 					*GetSuperview()										{ return( fSuperview ); }
			CP_Point					GetLocation()										{ return( fLocation ); }
			CP_Size						GetSize()											{ return( fSize ); }
	//@}

	//! @name Drawing
	//@{
												/*!
												*	Cause the View implementation to draw itself.
												*	
												*	@exception	none
												*	
												*	@return		void
												*/
	virtual	void						Draw()												{}
			void						HandleDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	virtual	void						PrepareDraw(
												CP_Canvas& /*inCanvas*/,
												CP_Region& /*inUpdateArea*/ )				{}
	virtual	CP_Canvas					SetupForDrawing();
	virtual void              			FinishDrawing( CP_Canvas& drawingCanvas );
												
	virtual	void						Invalidate( const CP_Region& /*inInvalidArea*/ )	{}
	virtual	void						Refresh()											{}

			CP_Point					GetOrigin()											{ return( fOrigin ); }
			void						SetOrigin( const CP_Point& inOrigin )				{ fOrigin = inOrigin; }
	//@}

	//! @name OwnerViewImp
	//@{
			CP_View_Imp 				*GetOwnerViewImp() const;
			void						SetOwnerViewImp( CP_View_Imp *inViewImp );
	//@}

	//! @name Getters
	//@{
	virtual	void						SetPortBounds( const CP_Rect& /*inBounds*/ )		{}
	//@}

	//! @name Positioning
	//@{
	virtual	void						Move( const CP_Point& /*inOffset*/ )				{}
	//@}

	//! @name Superview
	//@{
	virtual	void						AddSubview( CP_View_Imp */*inView*/ )				{}
	virtual	void						AdjustToSuperview( CP_View */*inView*/ )			{}
	//@}

	//! @name Positioning
	//@{
	virtual	void						SetLocation( const CP_Point& /*inLocation*/ )		{}
	virtual	void						SetSize( const CP_Size& /*inSize*/ )				{}
	//@}

	//! @name Testers
	//@{
	virtual	bool						IsVisible()	const									{ return( false ); }
	virtual	bool						IsActive() const									{ return( false ); }
	virtual	bool						IsEnabled() const									{ return( false ); }
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_View_Imp_Base();
										CP_View_Imp_Base(
												CP_View_Imp *inOwnerViewImp,
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inCreateImp = false );
	//@}
							
private:
	//! @name Initialization
	//@{
			void						CreateImp()											{}
	//@}

	// class data
protected:
	CP_View								*fSuperview;			/*! CP_View that contains this view */
	
	CP_Point							fOrigin;

	CP_View_Imp							*fOwnerViewImp;			/*! CP_View_Imp that we provide implementation for. */

	OSControl							fOSControl;				/*! Platform control to implement View features. */

	CP_EventHandler						fDummyEventHandler;		/*! Dummy event handler if the view imp has not been created. */
	
	CP_Point							fLocation;						/*! Upper left corner of view, relative to superview */
	CP_Size								fSize;							/*! Size of view */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
public:
	//! @name Setters
	//@{
										/*!
										*	Adjusts the passed in GtkWidget to be positioned correctly in the
										*	subview at the specified position, and resized to the passed in size.
										*	
										*	The implementation object of this view will handle the actual work.
										*
										*	@param		inGtkWidget			The GtkWidget that is being repositioned.
										*	@param		inSuperview			The CP_View that will contains this widget.
										*	@param		inLocation			Location in local coorinates releative to the passed
										*									in superview to place this view at.
										*	@param		inSize				The size to make the passed in GtkWidget, will correspond
										*									to the size of this view.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						SetSizeAndPosition(
												GtkWidget */*inGtkWidget*/,
												CP_View */*inSuperview*/,
												const CP_Point& /*inLocation*/,
												const CP_Size& /*inSize*/ )					{}
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_View_Imp_Base

